--[[
	UO_OGRANISER:
	'uo' (unOfficial) Version by Hoshizora [http://hoshizora.tripod.com/uo_ograniser]
	Original Concept and Program by modsyn [http://cs.ulm.edu/~jself]

	Uses LUA Player: 0.20 MOD 2 by Cools [http://www.theriddlingsphinx.com/]
	Last updated   : 2006.08.03
  ]]

----------------------
-- Import Libraries --
----------------------
require('lib/pad')
require('lib/string')

-----------------------
-- Utility Functions --
-----------------------
-- Hoshizora 2006.07.26: Load program images
function load_images()
	mainbg          =Image.load(imagesdir..'bg-main.png')
	contextbg       =Image.load(imagesdir..'bg-context.png')
	reportbg        =Image.load(imagesdir..'bg-report.png')
	padup           =Image.load(imagesdir..'pad-up.png')
	paddown         =Image.load(imagesdir..'pad-down.png')
	padleft         =Image.load(imagesdir..'pad-left.png')
	padright        =Image.load(imagesdir..'pad-right.png')
	padtriangle     =Image.load(imagesdir..'pad-triangle.png')
	padsquare       =Image.load(imagesdir..'pad-square.png')
	padcircle       =Image.load(imagesdir..'pad-circle.png')
	padcross        =Image.load(imagesdir..'pad-cross.png')
	padl            =Image.load(imagesdir..'pad-l.png')
	padr            =Image.load(imagesdir..'pad-r.png')
	padselect       =Image.load(imagesdir..'pad-select.png')
	padstart        =Image.load(imagesdir..'pad-start.png')
	analogleft      =Image.load(imagesdir..'analog-left.png')
	iconbattery     =Image.load(imagesdir..'icon-battery.png')
	iconlistlocked  =Image.load(imagesdir..'icon-listlocked.png')
	iconlistunlocked=Image.load(imagesdir..'icon-listunlocked.png')
	iconitemnoedit  =Image.load(imagesdir..'icon-itemnoedit.png')
	iconitemrepair  =Image.load(imagesdir..'icon-itemrepair.png')
	iconitemrepair0 =Image.load(imagesdir..'icon-itemrepair0.png')
	iconitemignore  =Image.load(imagesdir..'icon-itemignore.png')
	iconitemignore0 =Image.load(imagesdir..'icon-itemignore0.png')
	textformatted   =Image.load(imagesdir..'text-formatted.png')
	textoriginal    =Image.load(imagesdir..'text-original.png')
	text__sce       =Image.load(imagesdir..'text-__sce.png')
	textirshell     =Image.load(imagesdir..'text-irshell.png')
end

-- Hoshizora 2006.07.26: Pre-splash screen to initialise variables
function osPreSplashScreen()
	os:clear()
	os:print(fontdim,fontdim*28,'Initialising... please wait',black)
	os:print(fontdim,fontdim*31,msgSplashProgInfo1,black)
	os:print(fontdim,fontdim*32,msgSplashProgInfo2,black)
	os:print(fontdim*49,fontdim*32,pad_left_char(progver,10,' ',1),black)
	screen:blit(0,0,splashbg,0,0,splashbg:width(),splashbg:height(),false)
	screen:blit(0,0,os)
	screen.waitVblankStart()
	screen.flip()
end

-- Hoshizora 2006.07.13: New splash screen! :)
function osSplashScreen()
	msg=msgSplashProgInfo2
  while true do
		os:clear()
		os:print(fontdim,fontdim*28,'Press',black)
		os:print(fontdim,fontdim*31,msgSplashProgInfo1,black)
		os:print(fontdim,fontdim*32,msg,black)
		os:print(fontdim*49,fontdim*32,pad_left_char(progver,10,' ',1),black)
		screen:blit(0,0,splashbg,0,0,splashbg:width(),splashbg:height(),false)
		screen:blit(0,0,os)
		screen:blit(fontdim*7,fontdim*28,padstart)
		screen.waitVblankStart()
		screen.flip()

    Pad:Update()
		if Pad.Ax<(analogDeadZone*-1) then
			-- Pushed LEFT (Analog Stick)
			-- Hoshizora 2006.07.21: Added exit functionality to the splash screen
			ans=confirm(msgYNQuitOgraniser)
			if ans then System.Quit() -- Requires Lua Player .17dk_2 / .20 MOD and above to work
			else msg=msgSplashProgInfo2 end
    elseif Pad.tSt then
			break
		elseif Pad.tSl then
			-- Pushed Select
			-- Hoshizora 2006.07.20: Added screen capture functionality to the splash screen
			-- Hoshizora 2006.07.27: Added support for multiple screen captures
			msg=screen_save()
		end
	end
end

-- Hoshizora 2006.07.11: New ogranise screen with progress bar! :)
function osTouchProgressScreen(shortcutdir,curri,maxi,msState)
	local pbarwidth=300
	local pbarheight=fontdim+1
	local pbarcurrwidth=pbarwidth/maxi*(curri-1)
	local pbarpercent=math.floor((curri-1)/maxi*100)..'%'

	os:print(fontdim*11,fontdim*13,'Organising: '..pad_right(shortcutdir,26),black)
	os:print(fontdim*11,fontdim*14,'Completed '..curri-1 ..' out of '..maxi,black)
	os:print(fontdim*11,fontdim*19,msState,black)
	if msState=='Done.' then
		os:print(fontdim*11,fontdim*20,'Press   to view '..reporttitle..'.',black)
	else
		os:print(fontdim*11,fontdim*20,'Do not remove the Memory Stick.',black)
	end

	--Draw the progress bar!
	os:fillRect(fontdim*11+2,fontdim*16,pbarwidth,pbarheight+2,grey) -- Drop shadow!
	os:fillRect(fontdim*11,fontdim*16-2,pbarwidth,pbarheight+2,black) -- Outer border
	os:fillRect(fontdim*11+1,fontdim*16-1,pbarwidth-2,pbarheight) -- Leave 1 px for the border
	os:fillRect(fontdim*11+1,fontdim*16-1,pbarcurrwidth-2,pbarheight,green) -- The bar itself!
	os:print((screenwidth-fontdim*string.len(pbarpercent))/2,fontdim*16,pbarpercent,black) -- Percentage completed

	screen:blit(0,0,contextbg,0,0,contextbg:width(),contextbg:height(),false)
	osPrintBatteryLife()
	screen:blit(0,0,os)
	if msState=='Done.' then
		screen:blit(fontdim*17,fontdim*20,padcross)
	end
	screen.waitVblankStart()
	screen.flip()
end

-- Hoshizora 2006.07.07: Modified function touch_icon to support file counter
-- Hoshizora 2006.07.12: Rewrote the touch routine to support for multiple files within the shortcut dir
-- Hoshizora 2006.07.15: Support for directory renaming to repair/restore corrupted icons
-- Hoshizora 2006.07.22: Support for iR Shell folder name scheme
function touch_shortcut(proglist,list1_5item,repair,curri,maxi)
	local files={}
	local shortcutfiles={}
	local shortcutdir=list1_5item.name
	local repairThisItem=list1_5item.flag==1
	local oldProgDir=''
	local oldShortcutDir=''
	local newProgDir=''
	local sce='__SCE__'
	local irs='~1%'
	local scd='%'
	local success=true
	local errcode=''
	local inf,outf,msg,block,filedata,ok,prgdir,scdir,padlen

	local logfile=io.open(logfiledir,'a+') -- Log this operation
	if debug then logfile:write(pad_right('BEGIN: Shortcut '..shortcutdir,maxtitlelen)..'\n') end

	osClear()
	osTouchProgressScreen(shortcutdir,curri,maxi,'Reading...')
	files=System.listDirectory(gamedir..shortcutdir)
	for index,file in ipairs(files) do
		if not (file.directory or file.name=='.' or file.name=='..') then
			-- Read a file to memory
			inf,msg=io.open(gamedir..shortcutdir..'/'..file.name,'rb')
			if inf then
				block=''
				filedata={}
				repeat
					block=inf:read(1024*8)
					table.insert(filedata,block)
				until block==nil
				inf:close()
				table.insert(shortcutfiles,{name=file.name,data=filedata})

				-- Remove the file from MS after the read
				System.removeFile(gamedir..shortcutdir..'/'..file.name)
				if fileExists(gamedir..shortcutdir..'/'..file.name) then -- Assertion check
					if debug then logfile:write(pad_right('  System.removeFile('..file.name..')',maxtitlelen)..'  Error\n') end
					success=false
					errcode='A'
				end
			else
				if debug then logfile:write(pad_right('  io.open('..file.name..',rb): '..msg,maxtitlelen)..'  Error\n') end
				success=false
				errcode=errcode..'B'
			end
		end
	end

	-- Remove the shortcut directory (after all the files have been processed)
	System.removeDirectory(gamedir..shortcutdir)
	if dirExists(gamedir,shortcutdir)==1 then -- Assertion check
		if debug then logfile:write(pad_right('  System.removeDirectory()',maxtitlelen)..'  Error\n') end
		success=false
		errcode=errcode..'C'
	end

	-- Update the progress screen
	osClear()
	osTouchProgressScreen(shortcutdir,curri,maxi,'Saving...')

	-- Create the (new) shortcut directory, aka 'touching' the directory and modifying the timestamp
	-- Repair corrupted data icons using __SCE__foldername/%__SCE__foldername scheme
	if repair==1 and repairThisItem then
		ok=true
		oldShortcutDir=shortcutdir
		if (string.startsWith(shortcutdir,scd) or string.endsWith(shortcutdir,scd)) and not string.endsWith(shortcutdir,irs) then
			-- foldername/foldername% -=> __SCE__foldername/%__SCE__foldername
			-- Fixes %(_)sce(_)foldername and folder length errors
			ok=false
			scdir=string_format(shortcutdir,2)
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string.upper(proglist[i])==scdir then -- We have a match!
					oldProgDir=proglist[i]
					newProgDir=sce..fix_string_len(string_format(oldProgDir,0),maxSCEFolderNameLen-string.len(sce))
					shortcutdir=scd..newProgDir
					ok=true
					break
				end
			end
		elseif string.endsWith(shortcutdir,irs) then
			-- foldername                     _/folder~1% -=> __SCE__foldername/%__SCE__foldername
			ok=false
			scdir=string_format(shortcutdir,1)
			if string.len(scdir)>5 then scdir=string.sub(scdir,1,5) end
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string_format(proglist[i],6)==scdir then -- We have a match!
					oldProgDir=proglist[i]
					newProgDir=sce..fix_string_len(string_format(oldProgDir,3),maxSCEFolderNameLen-string.len(sce))
					shortcutdir='%'..newProgDir
					ok=true
					break
				end
			end
		else ok=false end
		if ok and string.upper(newProgDir)~=string.upper(oldProgDir) then --If newProgDir==oldProgDir, then the program dir is already repaired, so skip this section
			-- Let's do a check on the new dir names to make sure the names are unique and not already in use, if not unique, we'll skip this section
			if dirExists(gamedir,newProgDir)==1 or dirExists(gamedir,shortcutdir)==1 then ok=false
			else ok=true end
			if ok then
				-- The program dir simply needs to be renamed to match the shortcut dir name
				System.rename(gamedir..oldProgDir,gamedir..newProgDir) -- Requires Lua Player .20 and above to work
				if dirExists(gamedir,oldProgDir)==1 then -- Assertion check
					if debug then logfile:write(pad_right('  repair=1: System.rename()',maxtitlelen)..'  Error\n') end
					shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
					success=false
					errcode=errcode..'D'
				end
			else
				if debug then
					logfile:write(pad_right('  repair=1: Duplicate dir name exists:',maxtitlelen)..'  Error\n')
					logfile:write(pad_right('    '..newProgDir,maxtitlelen)..'  Error\n')
				end
				shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
				success=false
				errcode=errcode..'E'
			end
		elseif not ok then
			if debug then logfile:write(pad_right('  repair=1: program=shortcut not found',maxtitlelen)..'  Error\n') end
			shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
			success=false
			errcode=errcode..'F'
		end

	-- Repair corrupted data icons using foldername                     _/folder~1% scheme
	elseif repair==2 and repairThisItem then
		ok=true
		oldShortcutDir=shortcutdir
		if (string.startsWith(shortcutdir,scd) or string.endsWith(shortcutdir,scd)) and not string.endsWith(shortcutdir,irs) then
			-- __SCE__foldername/%__SCE__foldername -=> foldername                     _/folder~1%
			--               foldername/foldername% -=> foldername                     _/folder~1%
			ok=false
			scdir=string_format(shortcutdir,2)
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string.upper(proglist[i])==scdir then -- We have a match!
					oldProgDir=proglist[i]
					prgdir=string_format(oldProgDir,0)
					scdir=string_format(prgdir,7) -- Remove spaces between the folder names (if any)
					if string.len(scdir)>=(minShortcutDirSchemeLen-3) then
						newProgDir=pad_right_char(prgdir,minProgramDirSchemeLen,' ',0)..' 1'
						shortcutdir=string.sub(scdir,1,minShortcutDirSchemeLen-3)..irs
					else
						shortcutdir=pad_right_char(scdir,minShortcutDirSchemeLen-4,'_',1)..'_'..irs
						padlen=minShortcutDirSchemeLen-1-string.len(scdir)
						prgdir=prgdir..pad_right_char('',padlen,'_',0)
						newProgDir=pad_right_char(prgdir,minProgramDirSchemeLen-1,' ',0)..'1'
					end
					ok=true
					break
				end
			end
		elseif not (string.startsWith(shortcutdir,scd) or string.endsWith(shortcutdir,scd)) then ok=false end
		if ok and string.upper(newProgDir)~=string.upper(oldProgDir) then --If newProgDir==oldProgDir, then the program dir is already repaired, so skip this section
			-- Let's do a check on the new dir names first
			if dirExists(gamedir,newProgDir)==1 or dirExists(gamedir,shortcutdir)==1 then ok=false
			else ok=true end
			if ok then
				-- The program dir simply needs to be renamed
				System.rename(gamedir..oldProgDir,gamedir..newProgDir) -- Requires Lua Player .20 and above to work
				if dirExists(gamedir,oldProgDir)==1 then -- Assertion check
					if debug then logfile:write(pad_right('  repair=2: System.rename()',maxtitlelen)..'  Error\n') end
					shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
					success=false
					errcode=errcode..'G'
				end
			else
				if debug then
					logfile:write(pad_right('  repair=2: Duplicate dir name exists:',maxtitlelen)..'  Error\n')
					logfile:write(pad_right('    '..newProgDir,maxtitlelen)..'  Error\n')
				end
				shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
				success=false
				errcode=errcode..'H'
			end
		elseif not ok then
			if debug then logfile:write(pad_right('  repair=2: program=shortcut not found',maxtitlelen)..'  Error\n') end
			shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
			success=false
			errcode=errcode..'I'
		end
	
	-- Restore corrupted data icons using foldername/foldername% scheme
	elseif repair==3 and repairThisItem then
		ok=true
		oldShortcutDir=shortcutdir
		if (string.startsWith(shortcutdir,scd) or string.endsWith(shortcutdir,scd)) and not string.endsWith(shortcutdir,irs) then
			-- __SCE__foldername/%__SCE__foldername -=> foldername/foldername%
			-- Fixes folder length errors
			ok=false
			scdir=string_format(shortcutdir,2)
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string.upper(proglist[i])==scdir then -- We have a match!
					oldProgDir=proglist[i]
					newProgDir=fix_string_len(string_format(oldProgDir,0),maxFPFFolderNameLen)
					shortcutdir=newProgDir..scd
					ok=true
					break
				end
			end
		elseif string.endsWith(shortcutdir,irs) then
			-- foldername                     _/folder~1% -=> foldername/foldername%
			ok=false
			scdir=string_format(shortcutdir,1)
			if string.len(scdir)>5 then scdir=string.sub(scdir,1,5) end
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string_format(proglist[i],6)==scdir then -- We have a match!
					oldProgDir=proglist[i]
					newProgDir=fix_string_len(string_format(oldProgDir,3),maxFPFFolderNameLen)
					shortcutdir=newProgDir..scd
					ok=true
					break
				end
			end
		else ok=false end
		if ok and string.upper(newProgDir)~=string.upper(oldProgDir) then --If newProgDir==oldProgDir, then the program dir is already restored, so skip this section
			-- Let's do a check on the new dir names first
			if dirExists(gamedir,newProgDir)==1 or dirExists(gamedir,shortcutdir)==1 then	ok=false
			else ok=true end
			if ok then
				-- The program dir simply needs to be renamed
				System.rename(gamedir..oldProgDir,gamedir..newProgDir) -- Requires Lua Player .20 and above to work
				if dirExists(gamedir,oldProgDir)==1 then -- Assertion check
					if debug then logfile:write(pad_right('  repair=3: System.rename()',maxtitlelen)..'  Error\n') end
					shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
					success=false
					errcode=errcode..'J'
				end
			else
				if debug then
					logfile:write(pad_right('  repair=3: Duplicate dir name exists:',maxtitlelen)..'  Error\n')
					logfile:write(pad_right('    '..newProgDir,maxtitlelen)..'  Error\n')
				end
				shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
				success=false
				errcode=errcode..'K'
			end
		elseif not ok then
			if debug then logfile:write(pad_right('  repair=3: program=shortcut not found',maxtitlelen)..'  Error\n') end
			shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
			success=false
			errcode=errcode..'L'
		end
	end
	
	-- Create the shortcut dir using the original/fixed name
	if dirExists(gamedir,shortcutdir)==0 then -- Assertion check
		System.createDirectory(gamedir..shortcutdir)
		if dirExists(gamedir,shortcutdir)==0 then -- Assertion check
			if debug then logfile:write(pad_right('  System.createDirectory()',maxtitlelen)..'  Error\n')	end
			shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
			success=false
			errcode=errcode..'M'
			System.createDirectory(gamedir..shortcutdir)
			if dirExists(gamedir,shortcutdir)==0 then -- Assertion check
				if debug then logfile:write(pad_right('  System.createDirectory()',maxtitlelen)..'  Error\n')	end
				shortcutdir=gamedir -- Recover the files in game dir
				success=false
				errcode=errcode..'N'
			end
		end
	else
		if debug then
			logfile:write(pad_right('  Duplicate shortcut dir name exists:',maxtitlelen)..'  Error\n')
			logfile:write(pad_right('    '..shortcutdir,maxtitlelen)..'  Error\n')
		end
		shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
		success=false
		errcode=errcode..'O'
		System.createDirectory(gamedir..shortcutdir) -- Rename failed, so revert to old shortcut dir name
		if dirExists(gamedir,shortcutdir)==0 then -- Assertion check
			if debug then logfile:write(pad_right('  System.createDirectory()',maxtitlelen)..'  Error\n')	end
			shortcutdir=gamedir -- Recover the files in game dir
			success=false
			errcode=errcode..'P'
		end
	end

	-- Copy the files in memory back to the shortcut directory
	for index,file in ipairs(shortcutfiles) do
		outf,msg=io.open(gamedir..shortcutdir..'/'..file.name,'wb')
    if outf then
			for idx,dat in pairs(file.data) do
				outf:write(dat)
			end
			outf:flush()
			outf:close()
		else
			if debug then logfile:write(pad_right('  io.open('..file.name..',wb): '..msg,maxtitlelen)..'  Error\n') end
			success=false
			errcode=errcode..'Q'
		end
	end

	if debug then logfile:write(pad_right('END: Shortcut '..shortcutdir,maxtitlelen)..'  '..print_result(success)..'\n') end
	if not debug then
		msg=string_format(shortcutdir,0)
		if success then
			logfile:write(pad_right(msg,maxtitlelen)..'  OK\n')
		else
			logfile:write(pad_right(msg,maxtitlelen)..'  Error:'..errcode..'\n')
		end
	end

	if success then
		-- Hoshizora 2006.07.26: System.currentDirectory() in getProgDir() somehow failed to update the uo_Ograniser folder name path
		--                       after the touch operation, so here is the workaround
		if string.upper(oldProgDir..'/')==string.upper(progdirname) then
			progdirname=newProgDir..'/'
		end
	end

	logfile:flush()
	logfile:close()
  System.sleep(1000)

	-- Update the progress screen
	if curri==maxi then
		osClear()
		osTouchProgressScreen('',curri+1,maxi,'Done.')
		waitX()
	end
end

function print_result(ok)
	if ok then return 'OK' end
	return 'Failed'
end

function fix_string_len(str,len)
	if string.len(str)>len then -- return concatenated string
		return string.sub(str,1,len)
	end
	return str
end

-- Hoshizora 2006.07.22: Append x no. of chars to the right
function pad_right_char(str,maxlen,char,concat)
	local pad=''
	local strlen=string.len(str)
	if char==nil or string.len(char)~=1 then char=' ' end
	if maxlen==nil then maxlen=maxSCEFolderNameLen end
	if strlen<maxlen then
		for i=1,maxlen-strlen do
			pad=pad..char
		end
		return str..pad
	end
	if concat==1 then
		return fix_string_len(str,maxlen)
	end
	return str
end

-- Hoshizora 2006.07.22: Append x no. of chars to the right
function pad_left_char(str,maxlen,char,concat)
	local pad=''
	local strlen=string.len(str)
	if char==nil or string.len(char)~=1 then char=' ' end
	if maxlen==nil then maxlen=maxSCEFolderNameLen end
	if strlen<maxlen then
		for i=1,maxlen-strlen do
			pad=pad..char
		end
		return pad..str
	end
	if concat==1 then
		return fix_string_len(str,maxlen)
	end
	return str
end

-- Hoshizora 2006.07.13: Append white spaces to the right get a nicely formatted table effect
-- Hoshizora 2006.07.23: Code updated to use fix_string_len()
function pad_right(str,maxlen)
	local white=' '
	local pad=''
	local strlen=string.len(str)
	if maxlen==nil then maxlen=maxtitlelen end
	if strlen<maxlen then
		for i=1,maxlen-strlen do
			pad=pad..white
		end
		return str..pad
	end
	return fix_string_len(str,maxlen-2)..'..'
end

-- Hoshizora 2006.07.14: Append white spaces to the left to get a nicely formatted table effect
-- Hoshizora 2006.07.23: Code updated to use fix_string_len()
function pad_left(str,maxlen)
	local white=' '
	local pad=''
	local strlen=string.len(str)
	if maxlen==nil then maxlen=maxtitlelen end
	if strlen<maxlen then
		for i=1,maxlen-strlen do
			pad=pad..white
		end
		return pad..str
	end
	return fix_string_len(str,maxlen-2)..'..'
end

-- Hoshizora 2006.07.25: Get the program directory with trailing backslash (dynamic method)
function getProgramDir()
	local currdir=System.currentDirectory()
	local gamedirlen=string.len(gamedir)+1
	return string.sub(currdir,gamedirlen,string.indexOf(currdir,'/',gamedirlen))
end

-- Hoshizora 2006.07.25: Take and save multiple screenshots to your memory stick!
function screen_save()
	local maxn=9999
	local progdir=gamedir..progdirname
	local scrsdir=progdir..scrsavedirname..'/'
	local snapfile='Snap'..pad_left_char('1',4,'0',1)..'.png'
	local snapfile0='Snap'..pad_left_char('0',4,'0',1)..'.png'
	local result=dirExists(progdir,scrsavedirname)
	local msg
	
	if result==1 then
		for i=1,maxn do
			snapfile='Snap'..pad_left_char(i,4,'0',1)..'.png'
			msg=msgScreenCaptured..snapfile
			if not fileExists(scrsdir..snapfile) then break end -- We have found an empty file slot
			if i==maxn then
				snapfile=snapfile0  -- All file slots taken, so we will write to the default 'zero' file only
				msg=msgScreenCaptured..snapfile..' (no more slots)'
			end
		end
	elseif result==0 then
		System.createDirectory(scrsdir)
		msg=msgScreenCaptured..snapfile
		if dirExists(progdir,scrsavedirname)==0 then
			snapfile=nil
			msg=msgScrSaveError -- Something went wrong, we will not write the file to anywhere else
		end
	else -- result==-1
		snapfile=nil
		msg=msgScrSaveError -- Something went wrong, we will not write the file to anywhere else
	end
	if snapfile~=nil then
		screen:save(scrsdir..snapfile)
		return msg
	end
end

-- Hoshizora 2006.07.09: Remove Ogranise.log file (if exist)
function delLogFile()
	if fileExists(logfiledir) then
		System.removeFile(logfiledir)
	end
end

-- Hoshizora 2006.07.14: -- New report screen! :)
function osReportScreen()
	local lines={}
	local idx=1
	local ss=24
	local error=false
	local errcodepage=false
	local msg=msgReportScreen
	local inf,tlen,ans

	-- Read the log file to memory
	inf,msg=io.open(logfiledir,'r')
	if inf then
		msg=msgReportScreen
		for line in inf:lines() do
			if not error then
				if string.contains(line,'Error:') then
					msg=msgViewErrorCode
					error=true
				end
			end
			table.insert(lines,line)
		end
		inf:close()
	else
		if not debug then msg=pad_right(msg,maxStatusBarMsgLen) end
	end

	cprint_report(reporttitle,lines,idx,ss,msg)
	tlen=table.getn(lines)
  while true do
	  Pad:Update()
		if Pad.tLr and tlen>ss and not errcodepage then -- Use this only for multi-page report
			-- Pushed RIGHT
			if idx+ss<=tlen then -- Go to next page
				idx=idx+ss
			end
			cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
		elseif Pad.tLl and tlen>ss and not errcodepage then -- Use this only for multi-page report
			-- Pushed LEFT
			if idx-ss>=1 then -- Go to prev page
				idx=idx-ss
			end
			cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
		elseif Pad.tRr then
			-- Pushed CIRCLE
			errcodepage=false
			cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
		elseif Pad.tRd then
			-- Pushed CROSS
			errcodepage=true
			cprint_errorcodes()
		elseif Pad.Ax<(analogDeadZone*-1) then
			-- Pushed LEFT (Analog Stick)
			ans=confirm(msgYNQuitOgraniser)
			if ans then
				if not debug then delLogFile() end
				System.Quit() -- Requires Lua Player .17dk_2 / .20 MOD and above to work
			end
			if errcodepage then cprint_errorcodes()
			else cprint_report(reporttitle,lines,idx,ss,msgReportScreen) end
		elseif Pad.tSt then
			break
		elseif Pad.tSl then
			-- Hoshizora 2006.07.27: Added support for multiple screen captures
			msg=screen_save()
			if errcodepage then cprint_errorcodes(msg)
			else cprint_report(reporttitle,lines,idx,ss,msg) end
		end
	end
end

-- Hoshizora 2006.07.14: Helper function to print the report list
function cprint_report(title,lines,idx,ss,statusBarMsg)
	local tlen,totpages,currpage
  
	if statusBarMsg==nil then statusBarMsg='' end
	osClear()
  os:print(fontdim,fontdim,title,black)
	os:print(fontdim*3,fontdim*3,pad_right('Title',maxtitlelen)..'  Result',blue)
	tlen=table.getn(lines)
  for i=idx,(idx+ss-1) do
    if i>tlen then break end -- eof
		-- Hoshizora 2006.08.02: Print the lines using alternate colours for easy viewing
		if math.fmod(i,2)==1 then
			os:print(fontdim*3,fontdim*(i-idx+4),lines[i],white)
		else
			os:print(fontdim*3,fontdim*(i-idx+4),lines[i],ltblue)
		end
  end

	os:print(fontdim*3,fontdim*32,statusBarMsg,white)
  screen:blit(0,0,reportbg,0,0,reportbg:width(),reportbg:height(),false)
	osPrintBatteryLife()
	osPrintReportScreenPad(statusBarMsg)

	-- Get total no. of pages
	totpages=math.ceil(tlen/ss)
	currpage=math.ceil(idx/ss)
	if totpages>1 then
		os:print(fontdim*46,fontdim*29,pad_left('Page '..currpage..'/'..totpages,10),black)
	end
	if totpages>1 and currpage<=totpages and currpage~=1 then
		screen:blit(fontdim*57,fontdim*29,padleft)
	end		
	if totpages>1 and currpage>=1 and currpage~=totpages then
		screen:blit(fontdim*58,fontdim*29,padright)
	end

	screen:blit(0,0,os)
	screen.waitVblankStart()
  screen.flip()
end

-- Hoshizora 2006.08.03: On-screen Error Codes quick reference guide (accessible from the Summary Report screen)
function cprint_errorcodes(msg)
	if msg==nil then msg=msgErrorCodeScreen end
	osClear()
	os:print(fontdim,fontdim,errcodetitle,black)
	os:print(fontdim*3,fontdim*3, 'Code   Description',blue)
	os:print(fontdim*3,fontdim*4, 'A      Error in removing a file from Shortcut folder',white)
	os:print(fontdim*3,fontdim*5, 'B      Error in reading a file from Shortcut folder',ltblue)
	os:print(fontdim*3,fontdim*6, 'C      Error in removing the Shortcut folder',white)
	os:print(fontdim*3,fontdim*7, 'D,G,J  Error in renaming the Program folder',ltblue)
	os:print(fontdim*3,fontdim*8, 'E,H,K  Duplicate Program folder name exists',white)
	os:print(fontdim*3,fontdim*9, 'F,I,L  Unable to find Program:Shortcut folder pair',ltblue)
	os:print(fontdim*3,fontdim*10,'M,N,P  Error in creating the Shortcut folder',white)
	os:print(fontdim*3,fontdim*11,'O      Duplicate Shortcut folder name exists',ltblue)
	os:print(fontdim*3,fontdim*12,'Q      Error in writing a file to Shortcut folder',white)
	os:print(fontdim*3,fontdim*32,msg,white)
	screen:blit(0,0,reportbg,0,0,reportbg:width(),reportbg:height(),false)
	osPrintBatteryLife()
	osPrintReportScreenPad(msg)
	screen:blit(0,0,os)
	screen.waitVblankStart()
	screen.flip()
end

function osPrintReportScreenPad(msg)
	if msg==msgReportScreen then
		screen:blit(fontdim*9,fontdim*32,padstart)
		screen:blit(fontdim*43,fontdim*32,analogleft)
	elseif msg==msgViewErrorCode then
		screen:blit(fontdim*34,fontdim*32,padcross)
	elseif msg==msgErrorCodeScreen then
		screen:blit(fontdim*9,fontdim*32,padcircle)
		screen:blit(fontdim*42,fontdim*32,analogleft)		
	end
end

-- Hoshizora 2006.07.11: Recycle the table by removing all the elements in the table
function table.del(list)
	local n=table.getn(list)
	if n==0 then return end
	for i=1,n do
		table.remove(list)
	end
end

-- Hoshizora 2006.07.21: Updated to support iR Shell compatible folder naming scheme
function getGameList(prog1_5,list1_5,list1_0)
	local	files=System.listDirectory(gamedir)
	local tmplist={}
	local prgdir,strf

	table.del(prog1_5)
	table.del(list1_5)
	table.del(list1_0)
	-- 1st pass: Separate dir names with % in 1.5 list and those without in 1.0 list
	for index,file in ipairs(files) do
		if file.directory and not (file.name=='.' or file.name=='..') then
			if folder_scheme_match(file.name,'[ _]+[1_]$') then
				table.insert(prog1_5,file.name)
			elseif string.endsWith(file.name,'~1%') then
				table.insert(tmplist,{name=file.name,flag=1})
			elseif string.startsWith(file.name,'%') or string.endsWith(file.name,'%') then
				table.insert(list1_5,{name=file.name,flag=1})
			else
				table.insert(list1_0,{name=file.name,flag=1})
			end
		end
	end
	-- 2nd pass: Remove 1.5 titles in 1.0 list
	for i=1,table.getn(list1_5) do
		strf=string_format(list1_5[i].name,2)
		for j=1,table.getn(list1_0) do
			if strf==string.upper(list1_0[j].name) then -- We have a match!
				prgdir=table.remove(list1_0,j) -- Remove the 1.5 title in the 1.0 list
				table.insert(prog1_5,prgdir.name) -- Insert the 1.5 title into the 1.5 program list
				break -- We can exit the inner loop
			end
		end
	end
	for i=1,table.getn(tmplist) do -- Insert the remaining titles in temp list into the 1.5 list
		table.insert(list1_5,tmplist[i])
	end

	-- Sort the initial list in ascending order
	tableSortAsc(list1_0)
	tableSortAsc(list1_5)
end

function cprint_list(lines,st,ss,idx,hide)
	local tlen=table.getn(lines)
	local color

	for i=st,(st+ss) do
    if i>tlen then break end
    if i==idx then color=blue
    elseif math.fmod(i,2)==1 then color=white
		else color=ltblue end
		-- Hoshizora 2006.07.08: Pre-format homebrew titles before printing to screen
		if hide then
			os:print(fontdim*5,fontdim*(i-st+3),string_format(lines[i].name,0),color)
		else
			os:print(fontdim*5,fontdim*(i-st+3),lines[i].name,color)
		end
  end
end

function cprint_list_options(sh,repair,hide)
	if repair==1 then
		screen:blit(fontdim*39,fontdim*28+sh,text__sce)
	elseif repair==2 then
		screen:blit(fontdim*39,fontdim*28+sh,textirshell)
	elseif repair==3 then
		os:print(fontdim*39,fontdim*28+sh,'F&F%',yellow)
	else
		os:print(fontdim*39,fontdim*28+sh,'----',yellow)
	end
	if hide then
		screen:blit(fontdim*23,fontdim*28+sh,textformatted)
	else
		screen:blit(fontdim*23,fontdim*28+sh,textoriginal)
	end
end

function cprint_list_icons(lines,st,ss,repair,title)
	local tlen=table.getn(lines)
	local p

	for i=st,(st+ss) do
    if i>tlen then break end
		-- Hoshizora 2006.07.24: Icon to indicate whether the item will be repaired/ignored during organising
		if title==list1_0title then
			screen:blit(fontdim*3,fontdim*(i-st+3),iconitemnoedit)
		else
			if repair==0 then
				if lines[i].flag==1 then
					screen:blit(fontdim*3,fontdim*(i-st+3),iconitemrepair0)
				else
					screen:blit(fontdim*3,fontdim*(i-st+3),iconitemignore0)
				end
			else
				if lines[i].flag==1 then
					screen:blit(fontdim*3,fontdim*(i-st+3),iconitemrepair)
				else
					screen:blit(fontdim*3,fontdim*(i-st+3),iconitemignore)
				end
			end
		end
	end
end

-- Hoshizora 2006.07.08: Added status bar functionality. cprint() will do most of the GUI work for the Edit screen
function cprint(title,lines,idx,max,hide,repair,statusBarMsg)
	local ss=17 -- no. of items to display in the list+1
  local st=0
	local sh=-2

	-- Max isnt set then set it
  if max==1 then max=ss+1 end
	-- Up max if index is greater
  if idx>max then max=idx end
	-- Index got larger than max so set start point
  st=max-ss
	-- Adjust max when index goes low
  if idx<st then
    max=st+ss-1
    st=idx
  end
	-- Reset max when start is reset
  if st==1 then max=ss+1 end

	if statusBarMsg==nil then statusBarMsg='' end
	osClear()
	os:print(fontdim,fontdim,title,black)

  -- Print the list
	cprint_list(lines,st,ss,idx,hide)

	-- Hoshizora 2006.07.10: Updated screen instructions and display inner box as transparent
  os:print(fontdim*5,fontdim*24+sh,'   Manual Sort: Hold   +  /  On Selection',white)
	os:print(fontdim*5,fontdim*25+sh,'Automatic Sort:   Ascending/Descending Order',ltblue)
  os:print(fontdim*5,fontdim*26+sh,'Display Option:   FW1.0 List      FW1.5 List',white)
	os:print(fontdim*5,fontdim*27+sh,' Ignore Toggle:   All Titles      Single Title',ltblue)
	os:print(fontdim*5,fontdim*28+sh,'KXploit Option:          List          Name Scheme',white) 
	os:print(fontdim*5,fontdim*29+sh,'   Quit           Screen Capture          Organise',ltblue)
	os:print(fontdim*3,fontdim*32,statusBarMsg,white)
  screen:blit(0,0,mainbg,0,0,mainbg:width(),mainbg:height(),false)
	osPrintBatteryLife()
	osPrintListMode(title)
	cprint_list_icons(lines,st,ss,repair,title)
	cprint_list_options(sh,repair,hide)
	screen:blit(0,0,os)
	screen:blit(fontdim*26,fontdim*24+sh,padcross)
	screen:blit(fontdim*30,fontdim*24+sh,padup)
	screen:blit(fontdim*32,fontdim*24+sh,paddown)
	screen:blit(fontdim*21,fontdim*25+sh,padtriangle)
	screen:blit(fontdim*21,fontdim*26+sh,padleft)
	screen:blit(fontdim*37,fontdim*26+sh,padright)
	screen:blit(fontdim*21,fontdim*27+sh,padsquare)
	screen:blit(fontdim*37,fontdim*27+sh,padcircle)
	screen:blit(fontdim*21,fontdim*28+sh,padl)
	screen:blit(fontdim*37,fontdim*28+sh,padr)
	screen:blit(fontdim*5,fontdim*29+sh,analogleft)
	screen:blit(fontdim*19,fontdim*29+sh,padselect)
	screen:blit(fontdim*43,fontdim*29+sh,padstart)
	screen.waitVblankStart()
  screen.flip()

	return max
end

-- Hoshizora 2006.07.08: Added cprint2 function to decide which message to print in original/current listing
function cprint2(title,list,sel,max,move,hide,repair,msgOrig,msgCurr)
	if move then -- Current listing
		max=cprint(title,list,sel,max,hide,repair,msgCurr)
	else -- Original listing
		max=cprint(title,list,sel,max,hide,repair,msgOrig)
	end
	return max
end

function osPrintProgramInfo()
	-- Hoshizora 2006.07.08: Display program information
	local proginfo=progname .. ' ' .. progver
	os:print((screenwidth-fontdim*string.len(proginfo))/2,fontdim,proginfo,black)
end

function osPrintListMode(mode)
	local shx=5
	local shy=3
	if mode==list1_0title then
		screen:blit(fontdim*57+shx,fontdim*32-shy,iconlistlocked)
	else
		screen:blit(fontdim*57+shx,fontdim*32-shy,iconlistunlocked)
	end
end

function osPrintBatteryLife()
	-- Hoshizora 2006.07.16: Display battery life indicator
	local bbarwidth=13
	local bbarheight=8
	local bbarpercent=System.powerGetBatteryLifePercent()
	local bbareraserwidth=math.floor(bbarwidth/100*(100-bbarpercent))
	local icoshx=7
	local icoshy=-2
	local barshx=4
	local barshy=2
	
	--Draw the battery life bar!
	if bbarpercent>=50 then -- The battery life indicator
		os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbarwidth,bbarheight,green)
	elseif bbarpercent>=15 and bbarpercent<50 then
		os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbarwidth,bbarheight,yellow)
	else
		os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbarwidth,bbarheight,red)
	end
	os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbareraserwidth,bbarheight) -- The bar eraser!
	screen:blit(fontdim*56+icoshx,fontdim+icoshy,iconbattery)
end

-- Hoshizora 2006.07.11: Changed function name from clear() to osClear()
function osClear()
		-- Hoshizora 2006.07.10: Updated function to accomodate using image as background
		os:clear()
		osPrintProgramInfo()
end

-- Hoshizora 2006.07.10: Confirm screen before executing the touch process
function confirm(msg)
	local msgprompt='  Yes      No'

  while true do
		osClear()
		os:print((screenwidth-fontdim*string.len(msg))/2,fontdim*14,msg,black)
		os:print((screenwidth-fontdim*string.len(msgprompt))/2,fontdim*19,msgprompt,white)
    screen:blit(0,0,contextbg,0,0,contextbg:width(),contextbg:height(),false)
		osPrintBatteryLife()
		screen:blit(0,0,os)
		screen:blit((screenwidth-fontdim*string.len(msgprompt))/2,fontdim*19,padcross)
		screen:blit((screenwidth-fontdim*string.len(msgprompt))/2+fontdim*9,fontdim*19,padcircle)
		screen.waitVblankStart()
    screen.flip()
    Pad:Update()
    if Pad.tRd then
			-- Pushed CROSS
			osClear()
			return true
    elseif Pad.tRr then
			-- Pushed CIRCLE
			osClear()
			return false
    end
  end
end

function waitX()
  while true do
    Pad:Update()
    if Pad.Rd then break end
  end
end

function fileExists(file)
  local t,msg=io.open(file,"r")
	if not t then return false end
	t:close()
  return true
end

-- Hoshizora 2006.07.15: Check if directory exists
function dirExists(dir,name)
	local files=System.listDirectory(dir)
	local name1=string.upper(name)
	local result=0

	if files==nil then result=-1
	else
		for index,file in ipairs(files) do
			if file.directory and string.upper(file.name)==name1 then
				result=1
				break -- Dir exists, let's return the result
			end
		end
	end
	return result
end

function swap(table,i,j)
  local tmp=table[i]
  table[i]=table[j]
  table[j]=tmp
end

-- Hoshizora 2006.07.11: Added string_format function to preformat the string for the string.sort function
-- Hoshizora 2006.07.15: type=0-No case conversion, loose sc.disp; type=1-Case conversion, loose sc.cf; type=2-Case conversion, strict sc.cf-w/sce (for folder naming schemes 1-2)
-- Hoshizora 2006.07.19: type=3-No case conversion, loose prg.disp; type=4-Case conversion, loose prg.cf (for folder naming schemes 3-8)
-- Hoshizora 2006.07.21: type=5-No case conversion, compress prg.disp; type=6-Case conversion, compress prg.cf (for folder naming scheme 7, compress program folders)
--                       type 7-No case conversion, strip white.disp; type 8-Case conversion, strip white.cf
function string_format(str,type)
	local s=''
	local str1=''
	local shortcutPattern={'[ _]+[1_]$','~1%','[ _]*[1_]?~1%%$','%','^%%?_+[sS][cC][eE]_*'}

	if type==1 or type==2 or type==4 or type==6 or type==8 then -- Convert string to all caps
		s=string.upper(str)
	else s=str end -- No case conversion on the string

	slen=string.len(s)
	if string.endsWith(s,shortcutPattern[2]) then
		return string_format2(string.gsub(s,shortcutPattern[3],''))
	elseif string.endsWith(s,shortcutPattern[4]) then
		return string.sub(s,1,slen-1)
	elseif type==2 and string.startsWith(s,shortcutPattern[4]) then
		return string.sub(s,2,slen)
	elseif (type==3 or type==4) and folder_scheme_match(s,shortcutPattern[1]) then
		return string_format2(string.gsub(s,shortcutPattern[1],''))
	elseif (type==5 or type==6) then
		str1=string_format(string_format(s,3),7)
		if string.len(str1)>=minShortcutDirSchemeLen-4 then
			return string.sub(str1,1,minShortcutDirSchemeLen-4)
		else return str1 end
	elseif (type==7 or type==8) then
		return string.gsub(s,'%s','')
	else
		return string_format2(string.gsub(s,shortcutPattern[5],''))
	end
end

-- Hoshizora 2006.08.02 - Handle special folders named '_'/'_%'
function string_format2(s)
	if string.len(s)==0 then return '_' end
	return s
end

function folder_scheme_match(s,pattern)
	if string.len(s)<minProgramDirSchemeLen then return false end
	if string.match(s,pattern) then return true end
	return false	
end

function tableSortAsc(list)
	table.sort(list, function(a,b)
		return (string_format(a.name,1)<string_format(b.name,1))
	end)
end

function tableSortDesc(list)
	table.sort(list, function(a,b)
		return (string_format(a.name,1)>string_format(b.name,1))
	end)
end

function toggleItemRepair(list,i)
	local msg
	if list[i].flag==0 then
		list[i].flag=1
		msg=msgItemRepairToggle
	else
		list[i].flag=0
		msg=msgItemIgnoreToggle
	end
	return msg
end

function toggleAllItemRepair(list,repairall)
	local f
	if repairall then
		repairall=false
		msg=msgAllIgnoreToggle
		f=0
	else
		repairall=true
		msg=msgAllRepairToggle
		f=1
	end
	for i=1,table.getn(list) do
		list[i].flag=f
	end
	return repairall,msg
end

function msgOptionItemRepair(list,i)
	if list[i].flag==1 then return msgItemRepairOption end
  return msgItemIgnoreOption
end

function choose(title,list1_5,list1_0)
	local sel=1
	local tmp=''
	local show=list1_5
	local max=1
	local move=true
	local hide=true
	local repair=0
	local repairall=true
	local listorder=0
	local msg,ans

	osClear()
	max=cprint(title,show,sel,max,hide,repair,table.getn(list1_5)..msg1_5HomebrewsInTotal)

  while true do
	  Pad:Update()
		if Pad.tLd or Pad.Ay>analogDeadZone then
			-- Pushed DOWN (D-pad) or Pushed DOWN (Analog Stick)
			if sel<table.getn(show) then
				sel=sel+1
				msg=msgOptionItemRepair(show,sel)
				if Pad.Rd and move then
					-- Pushed CROSS
					swap(show,sel,sel-1)
					msg=msgManualSort
				end
			else -- We have reached the last item in the list (going down!)...
				sel=1
				msg=msgOptionItemRepair(show,sel)
				if Pad.Rd and move then
					-- Pushed CROSS
					tmp=table.remove(show) -- Remove the last item from list
					table.insert(show,1,tmp) -- So jump back to the first item on the next move!
					msg=msgManualSort
				end
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tLu or Pad.Ay<(analogDeadZone*-1) then
			-- Pushed UP (D-pad) or Pushed UP (Analog Stick)
			if sel>1 then
				sel=sel-1
				msg=msgOptionItemRepair(show,sel)
				if Pad.Rd and move then
					-- Pushed CROSS
					swap(show,sel,sel+1)
					msg=msgManualSort
				end
			else -- We have reached the first item in the list (going up!)...
				sel=table.getn(show)
				msg=msgOptionItemRepair(show,sel)
				if Pad.Rd and move then
					-- Pushed CROSS
					tmp=table.remove(show,1)
					table.insert(show,tmp) -- So jump to the last item on the next move!
					msg=msgManualSort
				end
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.Ax<(analogDeadZone*-1) then
			-- Pushed LEFT (Analog Stick)
			if Pad.Rd and move then
				-- Pushed CROSS
				debug=false -- (-oo-)
			end
			ans=confirm(msgYNQuitOgraniser)
			if ans then System.Quit() end -- Requires Lua Player .17dk_2 / .20 MOD and above to work
			max=cprint2(title,show,sel,max,move,hide,repair,table.getn(list1_0)..msg1_0HomebrewsInTotal,table.getn(list1_5)..msg1_5HomebrewsInTotal)

		elseif Pad.Ax>analogDeadZone then
			-- Pushed RIGHT (Analog Stick)
			if Pad.Rd and move then
				-- Pushed CROSS
				debug=true -- (^oo^)
			end

		elseif Pad.tLl then
			-- Pushed LEFT
			title=list1_0title
			show=list1_0
			move=false
			if sel>table.getn(show) then sel=1 end
			max=cprint(title,show,sel,max,hide,repair,table.getn(list1_0)..msg1_0HomebrewsInTotal)

		elseif Pad.tLr then
			-- Pushed RIGHT
			title=list1_5title
			show=list1_5
			move=true
			if sel>table.getn(show) then sel=1 end
			max=cprint(title,show,sel,max,hide,repair,table.getn(list1_5)..msg1_5HomebrewsInTotal)

		elseif Pad.tRu then
			-- Pushed TRIANGLE
			-- Hoshizora 2006.07.21: Button reassigned to perform sorting functions
			if move then -- Toggle mode
				listorder=math.fmod(listorder+1,2)
				if listorder==0 then
					-- Hoshizora 2006.07.07: Added SORT function, ASCENDING order
					tableSortAsc(list1_5)
					msg=msgAutoSort..'ascending order'
				elseif listorder==1 then
					-- Hoshizora 2006.07.07: Added SORT function, DESCENDING order
					tableSortDesc(list1_5)
					msg=msgAutoSort..'descending order'
				end
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tRl then
			-- Pushed SQUARE
			-- Hoshizora 2006.07.21: Button reassigned to perform all repair/ignore titles option
			if move then -- Toggle mode
				repairall,msg=toggleAllItemRepair(show,repairall)
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)
		
		elseif Pad.tRr then
			-- Pushed CIRCLE
			if move then
				-- Hoshizora 2006.07.24: Button reassigned to 'Repair this item/Ignore this item' toggle functionality
				msg=toggleItemRepair(show,sel)
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tRd then
			-- Pushed CROSS
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msgManualSort)

		elseif Pad.tL1 then
			-- Pushed L
			if move then -- Toggle mode
				if hide then
					msg=msgDispShowOption
					hide=false
				else
					msg=msgDispHideOption
					hide=true
				end
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tR1 then
			-- Pushed R
			repair=math.fmod(repair+1,4)
			if move then -- Toggle mode
				if     repair==0 then msg=msgIconNoChangeOption
				elseif repair==1 then msg=msgIconRepairSCEOption
				elseif repair==2 then msg=msgIconRepairIRSOption
				elseif repair==3 then msg=msgIconRestoreOption end
			end
			max=cprint2(title,show,sel,max,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tSl then
			-- Pushed Select
			-- Hoshizora 2006.07.08: Modified screen capture file save location
			-- Hoshizora 2006.07.27: Added support for multiple screen captures
			msg=screen_save()
			max=cprint(title,show,sel,max,hide,repair,msg)

		elseif Pad.tSt then
			-- Pushed Start
			-- Hoshizora 2006.07.08: Check battery life before commiting changes
			if System.powerIsLowBattery() and not System.powerIsPowerOnline() then
				max=cprint(title,show,sel,max,hide,repair,msgIsLowBattery)
			elseif title==list1_0title then
				max=cprint(title,show,sel,max,hide,repair,msgEditDisabled)
			else
				ans=confirm(msgYNCommitChanges)
				if ans then return list1_5,repair end
				max=cprint(title,show,sel,max,hide,repair,table.getn(list1_5)..msg1_5HomebrewsInTotal)
			end
		end
		osClear()
	end
end